﻿namespace WinFormsApp2;



//https://en.wikipedia.org/wiki/HRESULT
public struct HRESULT
{
    public static readonly int S_OK = 0;
    public static readonly int S_FALSE = -1;

    private uint value;
    private const uint CodeMask = 0b_00000000_00000000_11111111_11111111;
    private const uint FacilityMask = 0b_00000111_11111111_00000000_00000000;
    private const uint SeverityMask = 0b_10000000_00000000_00000000_00000000;
    private const uint ReservedRMask = 0b_01000000_00000000_00000000_00000000;
    private const uint CustomMask = 0b_00100000_00000000_00000000_00000000;
    private const uint ReservedNMask = 0b_00010000_00000000_00000000_00000000;
    private const uint ReservedXMask = 0b_00001000_00000000_00000000_00000000;

    public uint Code
    {
        get => value & CodeMask;
        set
        {
            this.value = this.value | value & CodeMask;
        }
    }

    public UI11 Facility
    {
        get => (value & FacilityMask) >> 16;
        set
        {
            this.value = this.value | ((uint)value & FacilityMask) << 16;
        }
    }

    public bool Severity
    {
        get => (value & SeverityMask) != 0;
        set
        {
            if (value)
            {
                this.value = this.value | SeverityMask;
            }
            else
            {
                this.value = this.value & ~SeverityMask;
            }
        }
    }

    public bool ReservedR
    {
        get => (value & ReservedRMask) != 0;
        set
        {
            if (value)
            {
                this.value = this.value | ReservedRMask;
            }
            else
            {
                this.value = this.value & ~ReservedRMask;
            }
        }
    }

    public bool Custom
    {
        get => (value & CustomMask) != 0;
        set
        {
            if (value)
            {
                this.value = this.value | CustomMask;
            }
            else
            {
                this.value = this.value & ~CustomMask;
            }
        }
    }

    public bool ReservedN
    {
        get => (value & ReservedNMask) != 0;
        set
        {
            if (value)
            {
                this.value = this.value | ReservedNMask;
            }
            else
            {
                this.value = this.value & ~ReservedNMask;
            }
        }
    }

    public bool ReservedX
    {
        get => (value & ReservedXMask) != 0;
        set
        {
            if (value)
            {
                this.value = this.value | ReservedXMask;
            }
            else
            {
                this.value = this.value & ~ReservedXMask;
            }
        }
    }



    public HRESULT(uint value)
    {
        this.value = value;
    }

    public HRESULT(int value)
    {
        this.value = (uint)value;
    }

    public HRESULT(HRESULT value)
    {
        this.value = value.value;
    }

    public static HRESULT FromWin32ErrorCode(SystemErrorCodes code)
    {
        return new HRESULT((uint)code & 0x0000FFFF | (uint)7 << 16 | 0x80000000);
    }

    public static explicit operator HRESULT(int value) => new HRESULT((uint)value);
    public static implicit operator HRESULT(uint value) => new HRESULT(value);
    public static explicit operator HRESULT(long value) => new HRESULT((uint)value);
    public static explicit operator HRESULT(ulong value) => new HRESULT((uint)value);

    public static explicit operator int(HRESULT hr) => (int)hr.value;
    public static implicit operator uint(HRESULT hr) => hr.value;
    public static implicit operator long(HRESULT hr) => hr.value;
    public static implicit operator ulong(HRESULT hr) => hr.value;

    public override string ToString()
    {
        return "0x" + value.ToString("x").ToUpper();
    }

}
